#!/usr/bin/perl
#
#  Simple WWW interface for Sleuth domain checker.
#  (c) 1999 Martin Mares <mj@ucw.cz>
#

#use lib '/home/mj/perl/lib';
use CGI;
use IO::Handle;
$query = new CGI;
print $query->header;

require 'check.conf';

$domain = $query->param('domain');
$server = $query->param('server');
$serverip = $query->param('serverip');
$verbose = $query->param('verbose');
$secondary = $query->param('secondary');
$submitter = $query->param('submitter');
$submit = "Submit";

print $query->start_html(	-title=>"DNS Sleuth",
				-author=>"$admin_email",
				-dtd=>'-//W3C//DTD HTML 4.0//EN'
			);
print "<H1>The DNS Sleuth</H1>\n";

if ($secondary_ns eq "") {
	$secondary = '';
}
if ($domain eq "") {
	print "<P>Sleuth is a tool for checking DNS zones for correctness and
for automatic processing of secondary name service requests.
<P>If you want to install it at your machine, it's available at <A HREF=\"ftp://atrey.karlin.mff.cuni.cz/pub/local/mj/net/\">our FTP server</A>.\n";
} elsif ($server eq "" && $serverip ne "") {
	print "<P>Please enter the real name of the name server you want to check the zone at.\n";
} elsif ($secondary ne "" && $serverip eq "") {
	print "<P>Please enter both name and IP address of your DNS server.\n";
} elsif ($secondary ne "" && $submitter eq "" && $need_submitter) {
	print "<P>Please fill in who is submitting the zone.\n";
} elsif ($server !~ /^[0-9A-Za-z.-]*$/ ||
	 $serverip !~ /^[0-9.]*$/ ||
	 $domain !~ /^[0-9A-Za-z.-]*$/ ||
	 $submitter !~ /^[0-9A-Za-z.\240-\376 -]*$/) {
	print "<P>Invalid characters encountered in parameters you've entered.\n";
} else {
	print "<H2>Check results for $domain</H2>\n";
	print "<P>\n";
	@c = "./sleuth";
	if ($verbose ne "") { push @c, "-v"; }
	push @c, "-h", $domain, $server;
	if ($serverip ne "") { push @c, $serverip; }
	if ($secondary ne "") { push @c, $secondary_ns, $secondary_ns_ip; }
	STDOUT->flush();
	$rc = system @c;
	if ($secondary ne "") {
		if ($rc / 256) {
			print "<P>Request for secondary DNS service rejected because of errors in the zone.\n";
			print "<P>If you have any questions, please ask $admin_ref.\n";
		} else {
			print "<P>Forwarding your secondary DNS service request to $admin_ref\n";
			print "who will notify you by e-mail sent to the hostmaster address given in the\n";
			print "SOA record when the secondary server will be configured.\n";
			shift @c;
			while ($c[0] =~ /^-[vh]$/) { shift @c; }
			$ru = $query->remote_user;
			($rh = $query->remote_host) =~ s/[^A-Za-z0-9. \240-\376-]/?/g;
			# This might look insecure, but we know all the names have been correctly
			# validated by the first sleuth run.
			`(	echo "Domain: $domain" ;
				echo "User: $ru" ;
				echo "From: $submitter ($rh)" ;
				echo ;
				./sleuth @c
			 ) | mail -s \"New zone\" $secondaries_to`;
			$? && print "<P><EM>Forwarding failed (rc=$?)</EM>\n";
		}
	}
}

print "<HR>\n";
print "<H2>New query</H2>\n";

print "<P>Please specify domain you wish to examine:\n
<UL><LI>If you wish to <EM>check</EM> the domain for correctness,
just enter its name.\n
<LI>If you wish to download the zone from a specific name server
instead of letting Sleuth choose one of the authoritative servers,
just specify its name. IP address of the server is needed only
in case it's inside the domain to be checked.\n";
if ($secondary_ns ne "") {
print "<LI>If you wish to request us to run a <EM>secondary</EM> server for
the zone, please include both the server name and its IP address and check
the \"Request secondary\" toggle. The
secondary server will run on <CODE>$secondary_ns</CODE>, so don't forget
to list it as an authoritative server in your zone file.\n";
}
print "</UL>\n";

print $query->startform;
print "<P><TABLE ALIGN=CENTER WIDTH=\"70\%\" BORDER=0>\n";
print "<TR><TD><EM>Domain:</EM><TD>", $query->textfield('domain'), "\n";
print "<TD><EM>Verbose output:</EM><TD>", $query->checkbox(-name=>'verbose',value=>'ON',-label=>''), "\n";
print "<TR><TD><EM>Server name:</EM><TD>", $query->textfield('server'), "\n";
if ($secondary_ns ne "") {
	print "<TD><EM>Request secondary:</EM><TD>", $query->checkbox(-name=>'secondary',value=>'ON',-label=>''), "\n";
}
print "<TR><TD><EM>Server IP:</EM><TD>", $query->textfield('serverip'), "\n";
if ($need_submitter) {
	print "<TD><EM>Your name:</EM><TD>", $query->textfield('submitter'), "\n";
}
print "</TABLE>\n";
print "<P>", $query->submit('action', 'Submit'), "\n";
print $query->endform;

print "<HR>\n";
print "<P>If you have any bug reports or suggestions, feel free to send them to $author_ref.\n";
print $query->end_html;
print "\n";
